@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-# Copyright (C) 2008 Peter Poeml / Novell Inc.  All rights reserved.
+# Copyright (C) 2008 Novell Inc.  All rights reserved.
 # This program is free software; it may be used, copied, modified
 # and distributed under the terms of the GNU General Public Licence,
 # either version 2, or (at your option) any later version.
@@ -75,10 +75,7 @@ def run(prg):
 
     except oscerr.LinkExpandError, e:
         print >>sys.stderr, 'Link cannot be expanded:\n', e
-        return 1
-
-    except oscerr.UnreadableFile, e:
-        print >>sys.stderr, e.msg
+        print >>sys.stderr, 'Use "osc repairlink" to fix merge conflicts.\n'
         return 1
 
     except (oscerr.NoWorkingCopy, oscerr.WorkingCopyWrongVersion), e:
@@ -142,3 +139,7 @@ def run(prg):
     except RPMError, e:
         print >>sys.stderr, e
         return 1
+
+    except OSError, e:
+        print >>sys.stderr, e
+        return 1
