@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-# Copyright (C) 2006 Peter Poeml / Novell Inc.  All rights reserved.
+# Copyright (C) 2006 Novell Inc.  All rights reserved.
 # This program is free software; it may be used, copied, modified
 # and distributed under the terms of the GNU General Public Licence,
 # either version 2, or (at your option) any later version.
@@ -21,7 +21,7 @@ try:
 except ImportError:
     import cElementTree as ET
 
-from conf import config
+from conf import config, cookiejar
 
 change_personality = {
             'i686':  'linux32',
@@ -31,21 +31,16 @@ change_personality = {
             's390':  's390',
         }
 
-change_personality_cross = {
-            'armv4l':   ' ',
-            'armv5el':  ' ',
-            'sh4':      ' ',
-        }
-
 can_also_build = { 
              'armv4l': [                                         'armv4l'                                 ],
              'armv5el':[                                         'armv4l', 'armv5el'                      ],
+             'armv7el':[                                         'armv4l', 'armv5el', 'armv7el'           ],
              's390x':  ['s390'                                                                            ],
              'ppc64':  [                        'ppc', 'ppc64',                                           ],
-             'i386':   [        'i586',                          'armv4l', 'armv5el',               'sh4' ],
-             'i586':   [                'i386',                  'armv4l', 'armv5el',               'sh4' ],
-             'i686':   [        'i586',                          'armv4l', 'armv5el',               'sh4' ],
-             'x86_64': ['i686', 'i586', 'i386',                  'armv4l', 'armv5el',               'sh4' ],
+             'i386':   [        'i586',                          'armv4l', 'armv5el', 'armv7el',    'sh4' ],
+             'i586':   [                'i386',                  'armv4l', 'armv5el', 'armv7el',    'sh4' ],
+             'i686':   [        'i586',                          'armv4l', 'armv5el', 'armv7el',    'sh4' ],
+             'x86_64': ['i686', 'i586', 'i386',                  'armv4l', 'armv5el', 'armv7el',    'sh4' ],
              }
 
 # real arch of this machine
@@ -57,7 +52,7 @@ if hostarch == 'i686': # FIXME
 class Buildinfo:
     """represent the contents of a buildinfo file"""
 
-    def __init__(self, filename, apiurl=config['apiurl']):
+    def __init__(self, filename, apiurl):
 
         try:
             tree = ET.parse(filename)
@@ -74,13 +69,8 @@ class Buildinfo:
             sys.stderr.write(error + '\n')
             sys.exit(1)
 
-        if apiurl.startswith('https://') or apiurl.startswith('http://'):
-            import urlparse
-            scheme, netloc, path = urlparse.urlsplit(apiurl)[0:3]
-            apisrv = netloc+path
-        else:
-            print >>sys.stderr, 'invalid protocol for the apiurl: \'%s\'' % apiurl
-            sys.exit(1)
+        if not (apiurl.startswith('https://') or apiurl.startswith('http://')):
+            raise urllib2.URLError('invalid protocol for the apiurl: \'%s\'' % apiurl)
 
         # are we building  .rpm or .deb?
         # need the right suffix for downloading
@@ -93,21 +83,24 @@ class Buildinfo:
                 break
 
         self.buildarch = root.find('arch').text
+        self.downloadurl = root.get('downloadurl')
+        self.debuginfo = 0
         if root.find('debuginfo') != None:
-            self.debuginfo = root.find('debuginfo').text
-        else:
-            self.debuginfo = 0
+            try:
+                self.debuginfo = int(root.find('debuginfo').text)
+            except ValueError:
+                pass
 
         self.deps = []
         for node in root.findall('bdep'):
             p = Pac(node,
                     self.buildarch,       # buildarch is used only for the URL to access the full tree...
                     self.pacsuffix,
-                    scheme,
-                    apisrv)
+                    apiurl)
                     
             self.deps.append(p)
 
+        self.vminstall_list = [ dep.name for dep in self.deps if dep.vminstall ]
         self.preinstall_list = [ dep.name for dep in self.deps if dep.preinstall ]
         self.runscripts_list = [ dep.name for dep in self.deps if dep.runscripts ]
 
@@ -131,24 +124,24 @@ class Pac:
 
     We build a map that's later used to fill our URL templates
     """
-    def __init__(self, node, 
-                 buildarch, pacsuffix, 
-                 scheme=config['scheme'], apisrv=config['apisrv']):
+    def __init__(self, node, buildarch, pacsuffix, apiurl):
 
         self.mp = {}
         for i in ['name', 'package', 
                   'version', 'release', 
                   'project', 'repository', 
-                  'preinstall', 'noinstall', 'runscripts',
+                  'preinstall', 'vminstall', 'noinstall', 'runscripts',
                  ]:
             self.mp[i] = node.get(i)
 
         self.mp['buildarch']  = buildarch
         self.mp['pacsuffix']  = pacsuffix
 
-
-        self.mp['repopackage'] = node.get('repopackage') or '_repository'
         self.mp['arch'] = node.get('arch') or self.mp['buildarch']
+
+	self.mp['extproject'] = node.get('project').replace(':', ':/')
+	self.mp['extrepository'] = node.get('repository').replace(':', ':/')
+        self.mp['repopackage'] = node.get('package') or '_repository'
         self.mp['repoarch'] = node.get('repoarch') or self.mp['arch']
 
         if not (self.mp['name'] and self.mp['arch'] and self.mp['version']):
@@ -156,12 +149,15 @@ class Pac:
                 "buildinfo for package %s/%s/%s is incomplete" 
                     % (self.mp['name'], self.mp['arch'], self.mp['version']))
 
-        self.mp['scheme'] = scheme
-        self.mp['apisrv'] = apisrv
+        self.mp['apiurl'] = apiurl
 
         self.filename = '%(name)s-%(version)s-%(release)s.%(arch)s.%(pacsuffix)s' % self.mp
 
         self.mp['filename'] = self.filename
+	if self.mp['repopackage'] == '_repository':
+	    self.mp['repofilename'] = self.mp['name']
+	else:
+	    self.mp['repofilename'] = self.mp['filename']
 
         # make the content of the dictionary accessible as class attributes
         self.__dict__.update(self.mp)
@@ -203,11 +199,15 @@ def get_built_files(pacdir, pactype):
         s_built = subprocess.Popen(['find', os.path.join(pacdir, 'SRPMS'), 
                                     '-name', '*.rpm'],
                                    stdout=subprocess.PIPE).stdout.read().strip()
+    elif pactype == 'kiwi':
+        b_built = subprocess.Popen(['find', os.path.join(pacdir, 'KIWI'), 
+                                    '-type', 'f'],
+                                   stdout=subprocess.PIPE).stdout.read().strip()
     else:
         b_built = subprocess.Popen(['find', os.path.join(pacdir, 'DEBS'),
                                     '-name', '*.deb'],
                                    stdout=subprocess.PIPE).stdout.read().strip()
-        s_built = None
+        s_built = ''
     return s_built, b_built
 
 
@@ -242,7 +242,7 @@ def main(opts, argv):
     repo = argv[0]
     arch = argv[1]
     build_descr = argv[2]
-    crossbuild = False
+    xp = []
 
     build_type = os.path.splitext(build_descr)[1][1:]
     if build_type not in ['spec', 'dsc', 'kiwi']:
@@ -263,20 +263,35 @@ def main(opts, argv):
         buildargs.append('--changelog')
     if opts.jobs:
         buildargs.append('--jobs %s' % opts.jobs)
+    if opts.icecream:
+        buildargs.append('--icecream %s' % opts.icecream)
+        xp.append('icecream')
+        xp.append('gcc-c++')
+    if opts.ccache:
+        buildargs.append('--ccache')
+        xp.append('ccache')
     if opts.baselibs:
         buildargs.append('--baselibs')
     if opts.debuginfo:
         buildargs.append('--debug')
+    if opts._with:
+        buildargs.append('--with %s' % opts._with)
+    if opts.without:
+        buildargs.append('--without %s' % opts.without)
+# FIXME: quoting
+#    if opts.define:
+#        buildargs.append('--define "%s"' % opts.define)
 
-    if opts.local_package:
-        pac = '_repository'
     if opts.alternative_project:
         prj = opts.alternative_project
         pac = '_repository'
         apiurl = config['apiurl']
     else:
         prj = store_read_project(os.curdir)
-        pac = store_read_package(os.curdir)
+        if opts.local_package:
+            pac = '_repository'
+        else:
+            pac = store_read_package(os.curdir)
         apiurl = store_read_apiurl(os.curdir)
 
     if not os.path.exists(build_descr):
@@ -293,7 +308,9 @@ def main(opts, argv):
                 print 'Overriding config value for %s=\'%s\' with \'%s\'' % (var, config[var], val)
             config[var] = val
 
-    config['build-root'] = config['build-root'] % {'repo': repo, 'arch': arch}
+    config['build-root'] = config['build-root'] % { 'repo': repo, 'arch': arch,
+                                                    'project' : prj, 'package' : pac
+                                                  }
 
     if not opts.extra_pkgs:
         extra_pkgs = config['extra-pkgs']
@@ -302,6 +319,10 @@ def main(opts, argv):
     else:
         extra_pkgs = opts.extra_pkgs
 
+    if xp:
+        extra_pkgs += xp
+
+
     print 'Getting buildinfo from server'
     bi_file = NamedTemporaryFile(suffix='.xml', prefix='buildinfo.', dir = '/tmp')
     try:
@@ -335,7 +356,7 @@ def main(opts, argv):
     bi_file.flush()
 
     bi = Buildinfo(bi_file.name, apiurl)
-    if bi.debuginfo:
+    if bi.debuginfo and not opts.disable_debuginfo:
         buildargs.append('--debug')
     buildargs = ' '.join(set(buildargs))
 
@@ -367,10 +388,27 @@ def main(opts, argv):
                 continue
 
     print 'Updating cache of required packages'
+
+    urllist = []
+
+    # transform 'url1, url2, url3' form into a list
+    if 'urllist' in config:
+        if type(config['urllist']) == str:
+	    re_clist = re.compile('[, ]+')
+            urllist = [ i.strip() for i in re_clist.split(config['urllist'].strip()) ]
+        else:
+            urllist = config['urllist']
+
+    # OBS 1.5 and before has no downloadurl defined in buildinfo
+    if bi.downloadurl:
+        urllist.append(bi.downloadurl + '/%(extproject)s/%(extrepository)s/%(arch)s/%(filename)s')
+    urllist.append( '%(apiurl)s/build/%(project)s/%(repository)s/%(buildarch)s/%(repopackage)s/%(repofilename)s' )
+
     fetcher = Fetcher(cachedir = config['packagecachedir'], 
-                      urllist = config['urllist'],
+                      urllist = urllist,
                       api_host_options = config['api_host_options'],
-                      http_debug = config['http_debug'])
+                      http_debug = config['http_debug'],
+                      cookiejar=cookiejar)
 
     # now update the package cache
     fetcher.run(bi)
@@ -383,7 +421,9 @@ def main(opts, argv):
         anyway, I assume... verifying package now saves time though, since we don't
         even try to set up the buildroot if it wouldn't work."""
 
-        if opts.no_verify:
+        if config['build-type'] == "xen" or config['build-type'] == "kvm":
+            print 'Skipping verification of package signatures due to secure VM build'
+        elif opts.no_verify:
             print 'Skipping verification of package signatures'
         else:
             print 'Verifying integrity of cached packages'
@@ -395,6 +435,7 @@ def main(opts, argv):
     rpmlist += [ '%s %s\n' % (i[0], i[1]) for i in rpmlist_prefers ]
 
     rpmlist.append('preinstall: ' + ' '.join(bi.preinstall_list) + '\n')
+    rpmlist.append('vminstall: ' + ' '.join(bi.vminstall_list) + '\n')
     rpmlist.append('runscripts: ' + ' '.join(bi.runscripts_list) + '\n')
 
     rpmlist_file = NamedTemporaryFile(prefix='rpmlist.', dir = '/tmp')
@@ -409,31 +450,28 @@ def main(opts, argv):
     bc_file.write(get_buildconfig(apiurl, prj, pac, repo, arch))
     bc_file.flush()
 
-
+    vm_options=""
+    if config['build-device'] and config['build-memory'] and config['build-type']:
+       if config['build-type'] == "kvm":
+          vm_options="--kvm " + config['build-device']
+       elif config['build-type'] == "xen":
+          vm_options="--xen " + config['build-device']
+       else:
+          print "ERROR: unknown VM is set ! (" + config['build-type'] + ")"
+          sys.exit(1)
+       if config['build-swap']:
+          vm_options+=" --swap " + config['build-swap']
+       if config['build-memory']:
+          vm_options+=" --memory " + config['build-memory']
+    
     print 'Running build'
-
-    # check for cross-build
-    if hostarch == 'x86_64':
-       if bi.buildarch == 'armv4l':
-          crossbuild = True
-       if bi.buildarch == 'armv5el':
-          crossbuild = True
-       if bi.buildarch == 'sh4':
-          crossbuild = True
-
-    if hostarch == 'i586':
-       if bi.buildarch == 'armv4l':
-          crossbuild = True
-       if bi.buildarch == 'armv5el':
-          crossbuild = True
-       if bi.buildarch == 'sh4':
-          crossbuild = True
-
-    cmd = '%s --root=%s --rpmlist=%s --dist=%s %s %s' \
+    cmd = '%s --root=%s --rpmlist=%s --dist=%s --arch=%s %s %s %s' \
                  % (config['build-cmd'],
                     config['build-root'],
                     rpmlist_file.name, 
                     bc_file.name, 
+                    bi.buildarch,
+                    vm_options,
                     build_descr, 
                     buildargs)
 
@@ -444,36 +482,31 @@ def main(opts, argv):
 
     # change personality, if needed
     cmd = tmpl % (config['su-wrapper'], cmd)
-    if crossbuild == False:
-       if hostarch != bi.buildarch:
-          cmd = change_personality[bi.buildarch] + ' ' + cmd
+    if hostarch != bi.buildarch:
+        cmd = (change_personality.get(bi.buildarch, '') + ' ' + cmd).strip()
 
-    if crossbuild == True:
-          cmd = change_personality_cross[bi.buildarch] + ' ' + cmd + ' --arch=%s' % (bi.buildarch)
-
-    print cmd
     rc = subprocess.call(cmd, shell=True)
     if rc: 
         print
         print 'The buildroot was:', config['build-root']
         sys.exit(rc)
 
-    pacdirlink = os.path.join(config['build-root'], '.build.packages')
-    if os.path.exists(pacdirlink):
-        pacdirlink = os.readlink(pacdirlink)
-        pacdir = os.path.join(config['build-root'], pacdirlink)
+    pacdir = os.path.join(config['build-root'], '.build.packages')
+    if os.path.islink(pacdir):
+        pacdir = os.readlink(pacdir)
+        pacdir = os.path.join(config['build-root'], pacdir)
 
-        if os.path.exists(pacdir):
-            (s_built, b_built) = get_built_files(pacdir, bi.pacsuffix)
+    if os.path.exists(pacdir):
+        (s_built, b_built) = get_built_files(pacdir, bi.pacsuffix)
+        
+        print
+        if s_built: print s_built
+        print
+        print b_built
 
-            print
-            if s_built: print s_built
-            print
-            print b_built
-
-            if opts.keep_pkgs:
-                for i in b_built.splitlines():
-                    import shutil
-                    shutil.copy2(i, os.path.join(opts.keep_pkgs, os.path.basename(i)))
+        if opts.keep_pkgs:
+            for i in b_built.splitlines() + s_built.splitlines():
+                import shutil
+                shutil.copy2(i, os.path.join(opts.keep_pkgs, os.path.basename(i)))
 
 
