@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-# Copyright (C) 2006 Peter Poeml / Novell Inc.  All rights reserved.
+# Copyright (C) 2006 Novell Inc.  All rights reserved.
 # This program is free software; it may be used, copied, modified
 # and distributed under the terms of the GNU General Public Licence,
 # either version 2, or (at your option) any later version.
@@ -23,7 +23,7 @@ def join_url(self, base_url, rel_url):
 
 
 class Fetcher:
-    def __init__(self, cachedir = '/tmp', api_host_options = {}, urllist = [], http_debug = False):
+    def __init__(self, cachedir = '/tmp', api_host_options = {}, urllist = [], http_debug = False, cookiejar = None):
 
         __version__ = '0.1'
         __user_agent__ = 'osbuild/%s' % __version__
@@ -42,10 +42,12 @@ class Fetcher:
         passmgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
         for host in api_host_options.keys():
             passmgr.add_password(None, host, api_host_options[host]['user'], api_host_options[host]['pass'])
-        authhandler = urllib2.HTTPBasicAuthHandler(passmgr)
+        openers = (urllib2.HTTPBasicAuthHandler(passmgr), )
+        if cookiejar:
+            openers += (urllib2.HTTPCookieProcessor(cookiejar), )
         self.gr = URLGrabber(user_agent=__user_agent__,
                             keepalive=1,
-                            opener = urllib2.build_opener(authhandler),
+                            opener = urllib2.build_opener(*openers),
                             progress_obj=self.progress_obj,
                             failure_callback=(self.failureReport,(),{}),
                             )
@@ -146,7 +148,7 @@ def verify_pacs(pac_list):
     saved_LC_ALL = os.environ.get('LC_ALL')
     os.environ['LC_ALL'] = 'en_EN'
 
-    o = subprocess.Popen(['/bin/rpm', '-K'] + pac_list, stdout=subprocess.PIPE,
+    o = subprocess.Popen(['rpm', '-K'] + pac_list, stdout=subprocess.PIPE,
                 stderr=subprocess.STDOUT, close_fds=True).stdout
 
     # restore locale
@@ -179,7 +181,9 @@ def verify_pacs(pac_list):
 
   Then, just start the build again.
 
-- If the key is unavailable, you may use --no-verify (which may pose a risk).
+- If you do not trust the packages, you should configure osc build for XEN or KVM
+
+- You may use --no-verify to skip the verification (which is a risk for your system).
 """ % {'name': missing_key, 
        'dir': os.path.expanduser('~')}
 
