@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-# Copyright (C) 2008 Peter Poeml / Novell Inc.  All rights reserved.
+# Copyright (C) 2008 Novell Inc.  All rights reserved.
 # This program is free software; it may be used, copied, modified
 # and distributed under the terms of the GNU General Public Licence,
 # either version 2, or (at your option) any later version.
@@ -74,11 +74,6 @@ class WorkingCopyOutdated(OscBaseError):
 class LinkExpandError(OscBaseError):
     """Exception raised when source link expansion fails"""
 
-class UnreadableFile(OscBaseError):
-    def __init__(self, msg):
-        OscBaseError.__init__(self)
-        self.msg = msg 
-
 class OscIOError(OscBaseError):
     def __init__(self, e, msg):
         OscBaseError.__init__(self)
